import stat
import os
from hashlib import sha256

from six.moves.urllib.parse import urlparse
import robosignatory.utils as utils

import logging
log = logging.getLogger("robosignatory.work")


def process_atomic(signer, ref, commitid, key, directory, doref=True):
    commitpath = os.path.join(directory, 'objects', commitid[:2],
                              commitid[2:] + '.commit')
    commitmetapath = commitpath + 'meta'

    if not os.path.exists(commitpath):
        log.info('Commit file at %s not found. Skipping' % commitpath)
        return
    if os.path.exists(commitmetapath):
        log.info('Commitmeta file at %s found. Skipping' % commitmetapath)
        return

    log.info('All checks passed, signing %s with %s' % (commitid, key))
    cmdline = signer.build_atomic_cmdline(key,
                                          commitid,
                                          commitpath,
                                          commitmetapath)
    log.info('Signing command line: %s' % cmdline)

    ret, stdout, stderr = utils.run_command(cmdline)
    if ret != 0:
        log.error('Error signing! Signing output: %s, stdout: %s, '
                  'stderr: %s', ret, stdout, stderr)
        return

    log.info('Fixing commitmeta file permissions')

    # Sigul writes it as 0600, which makes a lot of sense as a general file
    # mode for it, but this is just a signature file that we want published
    os.chmod(commitmetapath, (stat.S_IRUSR | stat.S_IWUSR | stat.S_IRGRP
                              | stat.S_IROTH))

    log.info('Commit was succesfully signed, writing ref %s' % ref)

    refpath = os.path.join(directory, 'refs', 'heads', ref)
    if os.path.exists(refpath):
        with open(refpath, 'r') as f:
            log.info('Previous commit for %s: %s'
                     % (ref, f.read().replace('\n', '')))

    dirname_refpath = os.path.dirname(refpath)
    if doref and not os.path.exists(dirname_refpath):
        log.info("Creating the directories for the refpath")
        os.makedirs(dirname_refpath)

    if doref:
        log.info('Writing %s to %s' % (commitid, refpath))

        with open(refpath, 'w') as f:
            f.write(commitid + '\n')

    log.info('Done')


def process_coreos(signer, key, bucket, tmpdir, artifact):
    filepath = urlparse(artifact["file"]).path.lstrip("/")
    local_filepath = os.path.join(tmpdir, os.path.basename(filepath))

    log.info("Downloading %s", filepath)
    bucket.download_file(filepath, local_filepath)

    log.info("Checking %s", filepath)
    hasher = sha256()
    with open(local_filepath, "rb") as f:
        while True:
            content = f.read(1024)
            if not content:
                break
            hasher.update(content)
    h = hasher.hexdigest()
    if h != artifact["checksum"]:
        log.error("Incorrect SHA256 for %s, not signing", filepath)
        return

    log.info("Signing %s", filepath)
    sig_filepath = local_filepath + ".sig"
    cmdline = signer.build_coreos_cmdline(
        key, local_filepath, sig_filepath)
    log.info('Signing command line: %s', cmdline)
    ret, stdout, stderr = utils.run_command(cmdline)
    if ret != 0:
        log.error('Error signing! Signing output: %s, stdout: %s, '
                  'stderr: %s', ret, stdout, stderr)
        return
    if not os.path.exists(sig_filepath):
        log.error("Signer did not produce any signature file for %s", filepath)
        return
    log.debug('Fixing signature file permissions')
    # Sigul writes it as 0600, which makes a lot of sense as a general file
    # mode for it, but this is just a signature file that we want published
    os.chmod(sig_filepath,
             (stat.S_IRUSR | stat.S_IWUSR | stat.S_IRGRP | stat.S_IROTH))

    log.info("Uploading signature for %s", filepath)
    bucket.upload_file(sig_filepath, filepath + ".sig")
    # Check the uploaded file
    uploaded = list(bucket.objects.filter(Prefix=filepath + ".sig"))
    if len(uploaded) != 1:
        log.warning("The signature for %s was not uploaded properly", filepath)
    elif uploaded[0].size != os.stat(sig_filepath).st_size:
        log.warning(
            "The uploaded signature for %s does not have the right size",
            filepath
        )

    os.remove(local_filepath)
    os.remove(sig_filepath)
